{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TemplateHaskell #-}
{-# LANGUAGE QuasiQuotes #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DeriveAnyClass #-}

module SEARX where

import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)

import Language.JavaScript.Inline

import System.Directory
import System.IO.Temp
import System.Process

import Servant.Client

import SEARX.Client

import Data.Foldable
import qualified Data.Text as T
import Data.Aeson
import GHC.Generics

{-
debug purpose, use getMetadataWith
getMetadataWith2 :: T.Text -> IO (Either ClientError Documents)
getMetadataWith2 q = do
  manager' <- newManager tlsManagerSettings
  runClientM
    (search (Just q) (Just 1) (Just 3) (Just "") (Just All) (Just Json))
    (mkClientEnv manager' $ BaseUrl Https "search.iscpif.fr" 443 "")

          read($url, function(err, article, meta) {
            // Title
            article.close();
-}

data Article =
  Article {title :: Maybe T.Text
          ,byline :: Maybe T.Text
          ,dir :: Maybe T.Text
          ,content :: Maybe T.Text
          ,textContent :: Maybe T.Text
          ,length :: Maybe Int
          ,excerpt :: Maybe T.Text
          ,siteName :: Maybe T.Text
          }
  deriving (Generic, Show, FromJSON, ToJSON)

setUpDirectory :: IO FilePath
setUpDirectory =
  do
    dir <- getCurrentDirectory
    _ <- withCurrentDirectory dir $
      traverse_
      callCommand
      [ "npm init --yes"
      , "npm install --save jsdom request-promise-native"
      , "git clone https://github.com/mozilla/readability.git"
      ]
    getCurrentDirectory


parseWebsite :: FilePath -> T.Text -> IO (Maybe Article)
parseWebsite tmpdir url =
  do
    withJSSession
      defJSSessionOpts {nodeWorkDir = Just tmpdir}
      [block|
            var {JSDOM} = require('jsdom');
            var rp = require('request-promise-native');
            var Readability = require('./readability');
            try {
                const val = await rp($url);
                const doc = new JSDOM(val, {url: $url})
                const reader = new Readability(doc.window.document);
                return reader.parse();
            } catch (err) {
                return null;
            }
            |]

specConcatEith :: Semigroup a => Either b a -> Either b a -> Either b a
specConcatEith (Left _) b = b
specConcatEith a (Left _) = a
specConcatEith (Right a) (Right b) = Right $ a <> b


getMetadataWith :: T.Text -> Int -> IO (Either ClientError Documents)
getMetadataWith q i = do
  manager' <- newManager tlsManagerSettings
  getMetadataWith' manager' q i 1
  where client' man = mkClientEnv man $ BaseUrl Https "search.iscpif.fr" 443 ""
        search' r' p' = search (Just r') (Just 1) (Just p') (Just "") (Just All) (Just Json)
        getMetadataWith' man req maxP curP
          | maxP >= curP = do
              val <- runClientM (search' req curP) (client' man)
              nextVal <- getMetadataWith' man req maxP (curP + 1)
              return $ specConcatEith val nextVal
          | otherwise = return (Right $ Documents 0 [])
