{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE TemplateHaskell #-}

module SEARX.Client where


import GHC.Generics
import Data.Aeson
import Control.Applicative ((<|>))

import Servant.API
import Servant.Client
import Data.Proxy (Proxy(..))

import qualified Data.Text as T
import qualified Control.Lens as L

data Document = Document
  {
    _document_id :: T.Text,
    _document_title :: Maybe T.Text,
    _document_abstract :: Maybe T.Text,
    _document_publicationDate :: Maybe T.Text,
    _document_sources :: [T.Text]
  } deriving (Show, Generic)
L.makeLenses ''Document

instance FromJSON Document where
  parseJSON (Object o) =
    Document <$>
    (o .: "url")
    <*> (o .:? "title")
    <*> (o .:? "content")
    <*> (o .:? "pubdate")
    <*> (o .: "engines" <|> pure [])

data Documents = Documents
  {
    _documents_total :: Int,
    _documents_hits :: [Document]
  } deriving (Show, Generic)
L.makeLenses ''Documents

instance Semigroup Documents where
  a <> b = Documents
    ((_documents_total a) + (_documents_total b))
    ((_documents_hits a) <> (_documents_hits b))

instance FromJSON Documents where
  parseJSON (Object o) =
    Documents <$> (o .: "number_of_results") <*> (o .: "results")


data Format = Json
  deriving (Show, Generic)

instance ToHttpApiData Format where
  toUrlPiece (Json) = "json"


data Language = All
  deriving (Show, Generic)

instance ToHttpApiData Language where
  toUrlPiece (All) = "all"


type Search =
  QueryParam "q" T.Text
  :> QueryParam "category_news" Int
  :> QueryParam "pageno" Int
  :> QueryParam "time_range" T.Text
  :> QueryParam "language" Language
  :> QueryParam "format" Format
  :> Post '[JSON] Documents

type SEARXAPI = Search

searxProxy :: Proxy (SEARXAPI)
searxProxy = Proxy

search :: Maybe T.Text -> Maybe Int -> Maybe Int -> Maybe T.Text -> Maybe Language -> Maybe Format -> ClientM Documents
search = client searxProxy
